/*
    \file   main.c

    Sine wave generation using DAC

    (c) 2018 Microchip Technology Inc. and its subsidiaries.

	DAC0 Demo - ATTiny3216, Sinus Ausgabe auf PA6

*/

/* 3.33 MHz (needed for delay) */
#define F_CPU                            (3333333UL)

/* VREF Startup time */
#define VREF_STARTUP_MICROS              (25)

/* Number of steps for a sine wave period */
#define SINE_WAVE_STEPS                  (100)

/* Sine wave amplitude */
#define SINE_AMPLITUDE                   (127)

/* Sine wave DC offset */
#define SINE_DC_OFFSET                   (128)

/* 2*PI */
#define M_2PI                            (2 * M_PI)

/* Frequency of the sine wave */
#define OUTPUT_FREQ                      (50)			// ca. in Hertz 10..100

/* Step delay for the synthesis loop */
#define STEP_DELAY_MICROS                ((1000000 / OUTPUT_FREQ) / SINE_WAVE_STEPS)

#include <avr/io.h>
#include <util/delay.h>
#include <math.h>

/* Buffer to store the sine wave samples */
uint8_t sineWave[SINE_WAVE_STEPS];

void sineWaveInit(void);
void VREF_init(void);
void DAC0_init(void);
void DAC0_setVal(uint8_t val);

void sineWaveInit(void)
{
    for(uint16_t i = 0; i < SINE_WAVE_STEPS; i++)
    {
        sineWave[i] = SINE_DC_OFFSET + SINE_AMPLITUDE * sin(i * M_2PI / SINE_WAVE_STEPS);
    }
}

void VREF_init(void)
{
    /* Voltage reference at 4.34V */
    VREF.CTRLA |= VREF_DAC0REFSEL_4V34_gc;
	
    /* DAC0/AC0 reference enable: enabled */
    VREF.CTRLB |= VREF_DAC0REFEN_bm;
	
    /* Wait VREF start-up time */
    _delay_us(VREF_STARTUP_MICROS);
}

void DAC0_init(void)
{
    /* Disable digital input buffer */
    PORTA.PIN6CTRL &= ~PORT_ISC_gm;
    PORTA.PIN6CTRL |= PORT_ISC_INPUT_DISABLE_gc;
	
    /* Disable pull-up resistor */
    PORTA.PIN6CTRL &= ~PORT_PULLUPEN_bm;
    
    /* default value */
    DAC0.DATA = SINE_DC_OFFSET;
	
    /* Enable DAC, Output Buffer, Run in Standby */
    DAC0.CTRLA = DAC_ENABLE_bm | DAC_OUTEN_bm | DAC_RUNSTDBY_bm;
}

void DAC0_setVal(uint8_t val)
{
    DAC0.DATA = val;
}

int main(void)
{
    uint16_t i = 0;

    VREF_init();
    DAC0_init();
    
    sineWaveInit();
    
    while (1)
    {
        DAC0_setVal(sineWave[i++]);    
        i = i % SINE_WAVE_STEPS;        
        _delay_us(STEP_DELAY_MICROS);
    }
}
