/*
 * ATTiny3216_Vcc_messen.cpp
 *
 * Created: 27.05.2020 12:59:55
 * Author : Micha
 *
 * 1. Vcc messen mit AtTiny3216 - 2. Messung eines Poti auf PortA6 - 3. Ausgabe der Werte auf USART
 *	1. eigene Betriebsspannung des ATTiny3216
 *	2. ber ADC
 *	3. mit USB zu TTL serieller Wandler, Anzeige auf PC mit Terminalprogramm
 */ 

// nderungen der vordefinierten Parameter in Visual Studio /Atmel Studio 7:
// Mit Maus Wort, z.B. ADC_REFSEL_VDDREF_g markieren, dann rechte Maustaste, dann
// "Goto Implementation" aufrufen (oder ALT-G). Im nun zu sehenden Text sind alternative
// Einstellungen zu sehen.

/* Atmel Studio 7 - nderung bei den Parametern des Linkers: 
  Zur Benutzung von sprintf() (u.a.) und float / double-Variablen - nderungen beim Linker:
- Men Projekt, Properties zu diesem Projekt aufrufen
- unter Toolchain / Linker / General die Option  "Use vprintf-library" anklicken
- unter Toolchain / Linker / Miscellaneous in das Feld eintragen: -lprintf_flt
- nun kann man sprintf() und printf() auch benutzen mit float und double
- Quelle: https://startingelectronics.org/articles/atmel-AVR-8-bit/print-float-atmel-studio-7/
-
*/

// Bitkram
//PORTA  |= (1 << bit);					// set Bit in PortA
//PORTA  &= ~(1 << bit);				// Reset Bit in PortA
//PORTB &= ~((1 << 0) | (1 << 2));		// lscht Bit 0 und 2 in PORTB
//PORTB ^= (1 << PB0);					// toggles Bit 0 in PORTB

// PORT aus Output;
//PORTA.DIRSET = PIN3_bm,		// Setze PORTA PIN3 als OUTPUT
//PORTB_set_port_dir(PIN2_bm, PORT_DIR_OUT);	/* Configure TX pin as an output (mit MyPort.h) */

// PORT als Input
//PORTB_set_port_dir(PIN3_bm, PORT_DIR_IN);		/* Configure RX pin as an input (mit MyPort.h) */
//PORTA.DIRCLR = PIN4_bm;		// PORTA.4 = INPUT

// Schreiben auf Port
//PORTA.OUTSET = PIN3_bm;		// PORTA.3 = HIGH
//PORTA_OUTCLR = PIN3_bm;		// PORTA.3 = LOW;

// Lesen von Port
//i = PORTA.IN;				// lese den ganzen Port A
//j = PORTA.IN & PIN4_bm;	// lese Bit 4 aus PORTA

//Toggle Port Pin
//PORTC.OUTTGL = PIN1_bm;				// Toggle PortC, Pin1

#ifndef F_CPU
#define F_CPU 3333333UL
#endif

#include <avr/io.h>
#include <util/delay.h>
#include "USART0.h"

// --------------------------------------------------------------------------------------
// Variablen ----------------------------------------------------------------------------
// --------------------------------------------------------------------------------------

uint16_t Vcc_raw = 0;
uint16_t Poti_raw = 0;

float Vcc_spg = 0;						/* measured Vcc value */;
float Poti_spg = 0;						// Spannunga Poti
char s[40];								// USART-Ausgabe


void CLKCTRL_init(void)
{
	#if (F_CPU == 1000000UL)                     /* 1MHz, 16/16, set fuse for 16 MHz */
	_PROTECTED_WRITE(CLKCTRL.MCLKCTRLB, CLKCTRL_PDIV_16X_gc | CLKCTRL_PEN_bm);
	#endif
	
	#if (F_CPU == 20000000UL)                  /* 20MHz, no division */
	_PROTECTED_WRITE(CLKCTRL.MCLKCTRLB, 0 << CLKCTRL_PEN_bp);
	#endif
	
	#if (F_CPU == 3333333UL)					/* 3.3 MHz bei Fuse = 20 MHz / 6 = 3.3 MHz*/
	_PROTECTED_WRITE(CLKCTRL.MCLKCTRLB, CLKCTRL_PDIV_6X_gc | CLKCTRL_PEN_bm);  /* default 3.33MHz (20MHz/6) set fuse for 20MHz */
	#endif
}


// ===========================================================================================
// ADC: ADC-Input ist PA6 (Poti)
// ===========================================================================================

void ADC0_init(void)
{
	/* Disable digital input buffer */
	PORTA.PIN6CTRL &= ~PORT_ISC_gm;					// Reset Bit
	PORTA.PIN6CTRL |= PORT_ISC_INPUT_DISABLE_gc;	// Set Bit
	
	/* Disable pull-up resistor */
	PORTA.PIN6CTRL &= ~PORT_PULLUPEN_bm;			// Reset Bit

	ADC0.CTRLA = 0 << ADC_ENABLE_bp;				// ADC Enable off - now change parameters

    VREF.CTRLA = VREF_ADC0REFSEL_0V55_gc;			/* Set the Vref back to default value 0.55V*/

	// Set SAMPCAP for correct ADC values...
	ADC0.CTRLC = ADC_PRESC_DIV256_gc				/* CLK_PER divided by 256 */
		| ADC_REFSEL_VDDREF_gc						/* VDD reference */
		| 1 << ADC_SAMPCAP_bp;						/* Important: Sample Capacitance Selection: enabled */;

	
	ADC0.CTRLA = ADC_ENABLE_bm						/* ADC Enable: enabled */
	    | 0 << ADC_FREERUN_bp						/* ADC Free run mode: enabled */
		| ADC_RESSEL_10BIT_gc;						/* 10-bit mode */
	
	/* Select ADC channel 6 */
	ADC0.MUXPOS  = ADC_MUXPOS_AIN6_gc;
}

uint16_t ADC0_read(void)
{
	/* Start ADC conversion */
	ADC0.COMMAND = ADC_STCONV_bm;
	
	/* Wait until ADC conversion done */
	while ( !(ADC0.INTFLAGS & ADC_RESRDY_bm) )
	{
		;
	}
	
	/* Clear the interrupt flag by writing 1: */
	ADC0.INTFLAGS = ADC_RESRDY_bm;
	
	return ADC0.RES;
}


float Vcc_messen(void)
{
	float spg = 0;
    VREF.CTRLA = VREF_ADC0REFSEL_1V1_gc;  /* Set the Vref to 1.1V*/

/* The following section is directly taken from Microchip App Note AN2447 page 13*/        
    ADC0.MUXPOS = ADC_MUXPOS_INTREF_gc    /* ADC internal reference, the Vbg*/;
    
    ADC0.CTRLC = ADC_PRESC_DIV256_gc        /* CLK_PER divided by 256 */
    | ADC_REFSEL_VDDREF_gc                /* Vdd (Vcc) be ADC reference */
    | 0 << ADC_SAMPCAP_bp                 /* Sample Capacitance Selection: disabled */;
    
    //float Vcc_value = 0                   /* measured Vcc value */;
    ADC0.CTRLA = 1 << ADC_ENABLE_bp       /* ADC Enable: enabled */
    | 1 << ADC_FREERUN_bp                 /* ADC Free run mode: enabled */
    | ADC_RESSEL_10BIT_gc                 /* 10-bit mode */;
    ADC0.COMMAND |= 1;                    // start running ADC
    while(!ADC0.INTFLAGS);				// wait until ADC result is ready
    //if (ADC0.INTFLAGS)                // if an ADC result is ready
    //{
    spg = ( 1024 * 1.1 ) / ADC0.RES;	/* calculate the Vcc value */
    //}
	return spg;
}

// ----------------------------------------------------------------------------------
// main  -----------------------------------------------------------------------------
// ----------------------------------------------------------------------------------


int main(void)
{
	CLKCTRL_init();
	USART0_init();								// 9600 Baud
	
	
    while (1) 
    {
		Vcc_spg = Vcc_messen();					// eigene Spannung des ATTiny3216 messen
		
		ADC0_init();
		Poti_raw = ADC0_read();
		Poti_spg = ( Vcc_spg / 1024) * Poti_raw;			// hier die gemessene VCC einsetzen!
		
		sprintf(s,"VCC: %f --- ", (double) Vcc_spg);			
		USART0_sendString((char*)s);				
		
		sprintf(s,"Poti: %f \r\n", (double) Poti_spg);
		USART0_sendString((char*)s);		
				
		_delay_ms(5000);
    }
}

