
#ifndef _i2c_twi_h
#define _i2c_twi_h
// ======================================================================================================
// I2C/TWI mit ATTiny3216
// ======================================================================================================
//
//	Wichtig: Bei TWI_Start(deviceAddr) ist jeweils die Adresse für Write oder Read anzugeben.
//				bei Write ist das Bit0 = 0,
//				bei Read ist das Bit0 = 1
//
#define TWI0_BAUD(F_SCL)      ((((float)F_CPU / (float)F_SCL)) - 10 )
//#define _24FC512_ADDRESS_W		0b10100000		// deviceaddr: Bit0 = 0 = Write
//#define _24FC512_ADDRESS_R		0b10100001		// deviceaddr: Bit0 = 1 = Read
#define ACK 1
#define NACK 0

void TWI_init()
{
	TWI0.MBAUD = (uint8_t)TWI0_BAUD(100000);	 // set MBAUD register for 100kHz
	TWI0.MCTRLA = 1 << TWI_ENABLE_bp			/* Enable TWI Master: enabled */
	| 0 << TWI_QCEN_bp							/* Quick Command Enable: disabled */
	| 0 << TWI_RIEN_bp							/* Read Interrupt Enable: disabled */
	| 1 << TWI_SMEN_bp							/* Smart Mode Enable: enabled */
	| TWI_TIMEOUT_DISABLED_gc					/* Bus Timeout Disabled */
	| 0 << TWI_WIEN_bp;							/* Write Interrupt Enable: disabled */

	TWI0.MCTRLB |= TWI_FLUSH_bm ;				/* Purge MADDR and MDATA */
	TWI0.MSTATUS |= TWI_BUSSTATE_IDLE_gc ;		        //Force TWI state machine into IDLE state
	TWI0.MSTATUS |= (TWI_RIF_bm | TWI_WIF_bm) ;	// Clear Interrupt Flags by writing 1 
}

// Wichtig: deviceaddr: Ist Bit0 = 1, dann folgt Read-Befehl, bei Bit0 = 0 folgt Write-Kommando
uint8_t TWI_Start(uint8_t deviceAddr)
{
	if ((TWI0.MSTATUS & TWI_BUSSTATE_gm) != TWI_BUSSTATE_BUSY_gc)		//Verify Bus is not busy
	{
		TWI0.MCTRLB &= ~(1 << TWI_ACKACT_bp);		// NACK
		TWI0.MADDR = deviceAddr;
		if (deviceAddr & 0x01)						//ist addressRead
		{
			while(!(TWI0.MSTATUS & TWI_RIF_bm))
			{
				; //USART0_sendString((char*)".\r\n");	// hier hängt es bei READ, wenn die I2C-Adresse nicht stimmt
			}
		}  
		else										//ist addressWrite	
		{
			while(!(TWI0.MSTATUS & TWI_WIF_bm)){;}
		} 
		return 0;
	}
	else return 1;	                                                        //Bus is busy
}

// mit ACK: TWI_Read(1), NAK: TWI_Read(0) <-- letztes zu lesendes Byte mit NAK lesen!
uint8_t TWI_Read(uint8_t ack)							// ACK=1 send ACK ; ACK=0 send NACK
{
	if ((TWI0.MSTATUS & TWI_BUSSTATE_gm) == TWI_BUSSTATE_OWNER_gc)		//Verify Master owns the bus
	{
		while(!(TWI0.MSTATUS & TWI_RIF_bm))			// Wait until RIF set
			{
				; //USART0_sendString((char*)"-\r\n");
			}				
		uint8_t data=TWI0.MDATA;
		if	(ack==1)	{TWI0.MCTRLB &= ~(1<<TWI_ACKACT_bp);}	// send ack
		else			{TWI0.MCTRLB |= (1<<TWI_ACKACT_bp);	}	// nack prepare for action stop

		return data ;
	}
	else
	return 1;	//Master does not own the bus

}

uint8_t TWI_Write(uint8_t write_data)
{
	if ((TWI0.MSTATUS & TWI_BUSSTATE_gm) == TWI_BUSSTATE_OWNER_gc)			                //Verify Master owns the bus
	{
		TWI0.MDATA = write_data;
		while (!((TWI0.MSTATUS & TWI_WIF_bm) | (TWI0.MSTATUS & TWI_RXACK_bm))) ;		//Wait until WIF set and RXACK cleared
		return 0;
	}
	else
	return 1;	//Master does not own the bus
}

void TWI_Stop(void)
{
	TWI0.MCTRLB |= TWI_ACKACT_NACK_gc;
	TWI0.MCTRLB |= TWI_MCMD_STOP_gc;
}

// ======== Beispielanwendung I2C Schreiben und Lesen ==================================================

//now the use of these routines :

//define HDC1080_ADDRESS_W        0b10000000		I2C-Adresse für das Schreiben
//#define HDC1080_ADDRESS_R        0b10000001		I2C-Adresse für das Lesen von Daten
//#define HDC1080_TEMP_REGISTER    0x00
//#define HDC1080_CONFIG_REGISTER    0x02
//
//static float temperature ;
//static float humidity ;
//
//void hdc1080_init() {
	//TWI_start(HDC1080_ADDRESS_W);
	//TWI_write(HDC1080_CONFIG_REGISTER);
	//TWI_write(0x10);                    // 0b0011 0000        0 : RST clear, 0 : reserved, 1 : heater enabled, 1 : Temp+Humid acquisition mode, 0 : battery status,
	////                    0 : Temp 14 bits, 00 : humid 14 bits,
	//TWI_write(0x00);                    // 0b0000 0000        [7:0] reserved, must be 0
	//TWI_STOP();
//}
//
//void hdc1080_read(){
	//TWI_start(HDC1080_ADDRESS_W);
	//TWI_write(HDC1080_TEMP_REGISTER);    // séquence pour démarrer une conversion
	//_delay_ms(20);

	//TWI_start(HDC1080_ADDRESS_R);
	//uint8_t temp_H = TWI_read(ACK);                    //  read avec ACK
	//uint8_t temp_L = TWI_read(ACK);                    //  read avec ACK
	//uint8_t humid_H = TWI_read(ACK);                    //  read avec ACK
	//uint8_t humid_L = TWI_read(NAK);                    //  read avec NACK
	//TWI_STOP();
//
//}
//
//The key point for several reads, is to use the last read  with with Nack : TWI_read(0) .
//Then you can issue a Stop.

// When the master is done receiving data from the slave, it must NAK the last data byte, 
// this informs the slave to release the bus so the master can send the stop!
// last TWI_Read(0); means Read with NAK.

// ======================================================================================================


////======================================================================================================
//// EEPROM
//// ======================================================================================================
//
//#define i2c_addr_24FC256 0xA0								// I2C-Adresse Microchip 24LC256 / 24FC256 eeprom
//uint16_t eeprom_address = 0;						// aktuelle Adresse im EEPROM
//
//// Ein Byte in externes EEPROM schreiben
//void writeEEPROM(uint8_t deviceaddress, uint16_t eeaddress, byte data )
//{
	//uint8_t result;
	//byte msb = (byte) (eeaddress >> 8);
	//byte lsb = (byte) (eeaddress & 0xFF);
	//
	//result = TWI_Start(deviceaddress);
	////if(result == 1) USART0_sendString((char*)"TWI-Start error\r\n");
	//
	//result = TWI_Write(msb);
	////if(result == 1) USART0_sendString((char*)"TWI-Write error\r\n");
//
	//result = TWI_Write(lsb);
	////if(result == 1) USART0_sendString((char*)"TWI-Write error\r\n");
	//
	//result = TWI_Write(data);
	////if(result == 1) USART0_sendString((char*)"TWI-Write error\r\n");
//
	//TWI_Stop();
//
	//_delay_ms(50);                                // Zeit für das EEPROM Daten zu schreiben
//}
//
//
//
//// Byte aus externem EEPROM lesen
//byte readEEPROM(uint8_t deviceaddress, uint16_t eeaddress)
//{
	//uint8_t result;
	//byte rdata = 255;
	//byte msb = (byte) (eeaddress >> 8);
	//byte lsb = (byte) (eeaddress & 0xFF);
	////Serial.print(msb); Serial.print(" - "); Serial.println(lsb);
	//
	//result = TWI_Start(deviceaddress);
	////if(result == 1) USART0_sendString((char*)"read: TWI-Start error\r\n");
	//
	//result = TWI_Write(msb);
	////if(result == 1) USART0_sendString((char*)"read: TWI-Write error\r\n");
	//
	//result = TWI_Write(lsb);
	////if(result == 1) USART0_sendString((char*)"read: TWI-Write error\r\n");
	//
	//result = TWI_Start(deviceaddress+1);				// Start mit Read-Bit (Bit0) gesetzt in deviceaddress
	////if(result == 1) USART0_sendString((char*)"read: TWI-Start error\r\n");
//
	//// bei mehreren zu lesenden Bytes wird jeweils mit TWI_Read(1) gelesen, bedeutet Lesen mit ACK
	//// nur beim letzten Byte schreibt man TWI_Read(0) mit NACK, dann gibt der Slave den Bus frei für TWI_Stop();	
	//rdata = TWI_Read(NACK);					// Read mit NAK (0), weil letztes zu lesendes Byte
	////if(rdata == 1) USART0_sendString((char*)"read: TWI-Read error\r\n");
		//
	//TWI_Stop();
	//return rdata;
//}


#endif
