/*
 * AtTiny3216_SPI_Test.cpp
 *
 * Created: 09.11.2019 12:05:29
 * Author : Micha
 *
 *	Test SPI anhand der 868-MHz-Funkmodule RF Hope RFM69W oder RFM69HW
 *
 * SPI luft, Schreiben in RFM69-Register und Lesen funktioniert, RFM69W sendet mit PA0 (PowerAmp0)
 * mit dem Modul RFM69HW luft es auch, dort wird jedoch PA1 (PowerAmp1) benutzt
 *
 * Senden funktioniert, Empfang auch, aber mit Abfrage des Flags Payload Ready, DIO0 / Pinchange PC3 nicht benutzt
 *
 *		ATTiny3216				RFM69(H)W
 *			3.3V					3.3V
 *			GND						GND
 *			PA3/SCK					SCK
 *			PA2/MISO				MISO
 *			PA1/MOSI				MOSI
 *			PA4/_SS					NSS
 *			PC3						DIO0
 *
 *		ATTiny3216				USB-to-UART-Adapter
 *			GND						GND
 *			PB2/TxD					RXD
 *
 *			PC0						LED
 */ 

// welches Funkmodul ist verbaut?
//#define _RFM69W_				// RFM69W mit PowerAmp PA0 und Leistung bis 13 dBm
#define _RFM69HW_				// leistungsstrkeres Modul mit PA1 und PA2, Leistung bis 20 dBm (100mW)

#ifndef F_CPU
#define F_CPU 3333333UL								// 3,3 MHz
#endif

#include <avr/io.h>
#include <util/delay.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <avr/interrupt.h>

#define byte uint8_t
#define word uint16_t

void SPI0_init(void);
void slaveSelect(void);
void slaveDeselect(void);
uint8_t SPI0_exchangeData(uint8_t data);
void USART0_sendString(char *str);
void setup(void);

void SPIWriteReg(word Wort);
unsigned int RFM69_ReadRegister(byte thisRegister, int bytesToRead);
void RFM69_WriteRegister(byte thisRegister, byte thisValue);


// "volatile" ist wichtig, da in ISR benutzt und sonst vom Compiler wegoptimiert
volatile bool rx_daten = false;				// zeigt empfangenen Interrupt an


void CLKCTRL_init(void)
{
	#if (F_CPU == 1000000UL)                     /* 1MHz, 16/16, set fuse for 16 MHz */
	_PROTECTED_WRITE(CLKCTRL.MCLKCTRLB, CLKCTRL_PDIV_16X_gc | CLKCTRL_PEN_bm);
	#endif
	
	#if (F_CPU == 20000000UL)                  /* 20MHz, no division */
	_PROTECTED_WRITE(CLKCTRL.MCLKCTRLB, 0 << CLKCTRL_PEN_bp);
	#endif
	
	#if (F_CPU == 3333333UL)					/* 3.3 MHz bei Fuse = 20 MHz`*/
	_PROTECTED_WRITE(CLKCTRL.MCLKCTRLA, CLKCTRL_CLKSEL_OSC20M_gc);
	_PROTECTED_WRITE(CLKCTRL.MCLKCTRLB, CLKCTRL_PDIV_6X_gc | CLKCTRL_PEN_bm);  /* default 3.33MHz (20MHz/6) set fuse for 20MHz */
	#endif

}

// schaltet die Main_CLK auf intern 3,33333 MHz, bei FUSE = 20 MHz / Prescaler = 6 (20 MHz / 6)
void Main_Clock_3_3_MHz(void)
{
	_PROTECTED_WRITE(CLKCTRL.MCLKCTRLA, CLKCTRL_CLKSEL_OSC20M_gc);
	_PROTECTED_WRITE(CLKCTRL.MCLKCTRLB, CLKCTRL_PDIV_6X_gc | CLKCTRL_PEN_bm);  /* default 3.33MHz (20MHz/6) set fuse for 20MHz */

}

// schaltet die MAIN_CLK auf internen Oszillator 32,768 kHz
void Main_Clock_32_kHz(void)
{
	_PROTECTED_WRITE(CLKCTRL.MCLKCTRLA, CLKCTRL_CLKSEL_OSCULP32K_gc);	// MAIN_CLK = 32,768 kHz
	_PROTECTED_WRITE(CLKCTRL.MCLKCTRLB, 0 << CLKCTRL_PEN_bp);	// Prescaler aus
}

// ==================================================================================
// SPI0:
// ==================================================================================

void SPI0_init(void)
{
	//PORTA.DIRSET = PIN1_bm;			// MOSI = Output
	//PORTA.DIRCLR = PIN2_bm;			// MISO = Input
	//PORTA.DIRSET = PIN3_bm;			// SCK = Output
	//PORTA.DIRSET = PIN4_bm;			// SS = Output
	
	PORTA.DIR |= PIN1_bm;				/* Set MOSI pin direction to output */
	PORTA.DIR &= ~PIN2_bm;				/* Set MISO pin direction to input */
	PORTA.DIR |= PIN3_bm;				/* Set SCK pin direction to output */
	PORTA.DIR |= PIN4_bm;				/* Set SS pin direction to output */
	
	SPI0.CTRLA = (0 << SPI_CLK2X_bp)		/* Enable Double Speed: disabled */
	| (0 << SPI_DORD_bp)					/* Data Order Setting: (1) = MSB first, RFM69 = LSB first (0) */
	| (1 << SPI_MASTER_bp)				/* SPI module in master mode */
	| SPI_PRESC_DIV16_gc;				/* System Clock / 16 */
// ergibt sich durch eine falsche Setzung des SPI-Prescalers eine "falsche" Frequenz, 
// erzeugt SPI kein SCK-Signal mehr....

	SPI0.CTRLB = (0 << SPI_BUFEN_bp)			/* Buffer Mode Enable: disabled (0) */
			 | (0 << SPI_BUFWR_bp)				/* Buffer Write Mode: disabled */
			 | SPI_MODE_0_gc					/* SPI Mode 0 (default) */
			 | (1 << SPI_SSD_bp);				/* Slave Select Disable: yes (1) */ 

	SPI0.INTCTRL = 0 << SPI_DREIE_bp    /* Data Register Empty Interrupt Enable: disabled */
			| 1 << SPI_IE_bp					/* Interrupt Enable: enabled */
			| 0 << SPI_RXCIE_bp					/* Receive Complete Interrupt Enable: 1 = enabled, 0 = disabled */
			| 0 << SPI_SSIE_bp					/* Slave Select Trigger Interrupt Enable: disabled */
			| 0 << SPI_TXCIE_bp;				/* Transfer Complete Interrupt Enable: disabled */

	SPI0.CTRLA |= (1 << SPI_ENABLE_bp);				/* Set SPI Enable                                                                                                                                        SPI Module */

}

/**
 * \brief Enable SPI_0
 * 1. If supported by the clock system, enables the clock to the SPI
 * 2. Enables the SPI module by setting the enable-bit in the SPI control register
 *
 * \return Nothing
 */
void SPI0_enable()
{
	SPI0.CTRLA |= SPI_ENABLE_bm;
}

/**
 * \brief Disable SPI_0
 * 1. Disables the SPI module by clearing the enable-bit in the SPI control register
 * 2. If supported by the clock system, disables the clock to the SPI
 *
 * \return Nothing
 */
void SPI0_disable()
{
	SPI0.CTRLA &= ~SPI_ENABLE_bm;
}


uint8_t SPI0_exchangeData(uint8_t data)
{
	SPI0.DATA = data;
	//_delay_us(20);
	while (!(SPI0.INTFLAGS & SPI_IF_bm))		/* waits until data is exchanged*/
	{
		; //USART0_sendString( (char *) "\r\nWaiting...\r\n");
	}
	//USART0_sendString( (char *) "\r\n Leave ExchangeData\r\n");
	return SPI0.DATA;
}

void slaveSelect(void)
{
	PORTA.OUT &= ~PIN4_bm; // Set SS pin value to LOW - PORTA.4 ist SS
}

void slaveDeselect(void)
{
	PORTA.OUT |= PIN4_bm; // Set SS pin value to HIGH - PORTA.4 ist SS
}
// ===========================================================================================



// ===========================================================================================
// USART0:
// ===========================================================================================
#define USART0_BAUD_RATE(BAUD_RATE) ((float)(F_CPU * 64 / (16 * (float)BAUD_RATE)) + 0.5)

void USART0_init(void)
{

	PORTB.DIRCLR = PIN3_bm;		// PORTB.3 = INPUT = RxD
	PORTB.DIRSET = PIN2_bm,		// PORTB.2 = OUTPUT = TxD

	USART0.BAUD = (uint16_t)USART0_BAUD_RATE(9600);				// bei F_CPU = 1 Mhz oder 3,3 MHz
	//USART0.BAUD = (uint16_t)USART0_BAUD_RATE(115200);
	
	USART0.CTRLB |= USART_TXEN_bm;          /* Transmitter Enable bit mask. */

}

static void USART0_sendChar(char c)
{
	while (!(USART0.STATUS & USART_DREIF_bm))
	{
		;                                   /* Wait for USART ready for receiving next char */
	}
	USART0.TXDATAL = c;
}

//Anwendung: USART0_sendString( (char *) "-Text-\r\n");
void USART0_sendString(char *str)
{
	uint8_t i;
	for(i = 0; i < strlen(str); i++)
	{
		USART0_sendChar(str[i]);
	}
}

// Debugging: eine Variable anzeigen
void debug_int(uint16_t wert)
{
	uint8_t i= 0;
	char s[10] = "";
	// char * itoa(int val,char * s,int radix)
	itoa(wert,s,10);
	for(uint8_t i=0;i<9;i++)			// CR an die Ziffern anhngen
	{
		if (s[i] == 0)
		{
			s[i]= 13;
			s[i+1] = 10;
			break;
		}
	}
	do
	{
		USART0_sendChar(s[i]);	// wait until TX can send
	} while (s[++i] != 0);
}
// =======================================================================================


// =======================================================================================
// Ports
// =======================================================================================

enum port_pull_mode { PORT_PULL_OFF, PORT_PULL_UP, };
enum port_dir { PORT_DIR_IN, PORT_DIR_OUT, PORT_DIR_OFF, };

/**
 * \brief Set port data direction
 *
 * Select if the pin data direction is input, output or disabled.
 * If disabled state is not possible, this function throws an assert.
 *
 * \param[in] mask      Bit mask where 1 means apply direction setting to the
 *                      corresponding pin
 * \param[in] dir       PORT_DIR_IN  = Data direction in
 *                      PORT_DIR_OUT = Data direction out
 *                      PORT_DIR_OFF = Disables the pin
 *                      (low power state)
 */
  void PORTC_set_port_dir(const uint8_t mask, const enum port_dir dir)
{
	switch (dir) {
	case PORT_DIR_IN:
		VPORTA.DIR &= ~mask;
		break;
	case PORT_DIR_OUT:
		VPORTA.DIR |= mask;
		break;
	case PORT_DIR_OFF:
		/*/ should activate the pullup for power saving
		  but a bit costly to do it here */
		{
			for (uint8_t i = 0; i < 8; i++) {
				if (mask & 1 << i) {
					*((uint8_t *)&PORTC + 0x10 + i) |= 1 << PORT_PULLUPEN_bp;
				}
			}
		}
		break;
	default:
		break;
	}
}


/**
 * \brief Set port pin pull mode
 *
 * Configure pin to pull up, down or disable pull mode, supported pull modes are defined by device used
 *
 * \param[in] pin       The pin number within port
 * \param[in] pull_mode Pin pull mode
 */
  void PORTC_set_pin_pull_mode(const uint8_t pin, const enum port_pull_mode pull_mode)
{
	volatile uint8_t *port_pin_ctrl = ((uint8_t *)&PORTC + 0x10 + pin);

	if (pull_mode == PORT_PULL_UP) {
		*port_pin_ctrl |= PORT_PULLUPEN_bm;
	} else if (pull_mode == PORT_PULL_OFF) {
		*port_pin_ctrl &= ~PORT_PULLUPEN_bm;
	}
}


/**
 * \brief Set port pin input/sense configuration
 *
 * Enable/disable digital input buffer and pin change interrupt,
 * select pin interrupt edge/level sensing mode
 *
 * \param[in] pin pin number within port
 * \param[in] isc PORT_ISC_INTDISABLE_gc    = Interrupt disabled but input buffer enabled
 *                PORT_ISC_BOTHEDGES_gc     = Sense Both Edges
 *                PORT_ISC_RISING_gc        = Sense Rising Edge
 *                PORT_ISC_FALLING_gc       = Sense Falling Edge
 *                PORT_ISC_INPUT_DISABLE_gc = Digital Input Buffer disabled
 *                PORT_ISC_LEVEL_gc         = Sense low Level
 */
void PORTC_pin_set_isc(const uint8_t pin, const PORT_ISC_t isc)
{
	volatile uint8_t *port_pin_ctrl = ((uint8_t *)&PORTC + 0x10 + pin);

	*port_pin_ctrl = (*port_pin_ctrl & ~PORT_ISC_gm) | isc;
}


// =======================================================================================



// =======================================================================================
// RFM69(H)W
// =======================================================================================

const byte RFM69_SendData[] = {"Testsendung RFM69 868\r\n"};
byte RFM69_RxData[] =   {"xxxxxxxxxxxxxxxxxxxxxxx"};

// Masken fr SPI
const byte READ =  0b00000000;   // read command MSB = 0 (RFM65W - MSB ist Null)
const byte WRITE = 0b10000000;   // write command MSB = 1 (RFM65W - MSB ist 1)

unsigned int gelesen;
unsigned long frequenz;
bool rx = false;                            // zeigt ankommende Daten an (FIFO hat Daten)

#define  FREQ_SEL 3                         // 869.350 MHz
// Frequenzen:
// 868 MHz = D90000 Register FrfMSB = D9, Reg. FrfMid = 00, Reg FrfLSB = 00
// 869,350 MHz = D95666 Register FrfMSB = D9, Reg. FrfMid = 58, Reg FrfLSB = 66
// 869,400 MHz = D95999 Register FrfMSB = D9, Reg. FrfMid = 59, Reg FrfLSB = 99
// 915 MHz = E4C000 Register FrfMSB = E4, Reg. FrfMid = C0, Reg FrfLSB = 00
// die Werte werden in das Register 07, 08, 09 geschrieben:
const word RFM69FreqTbl[6][3] =
{
	{0x074e, 0x08c0, 0x0900},  //315MHz
	{0x076c, 0x0880, 0x0900},  //434MHz
	{0x07d9, 0x0800, 0x0900},  //868MHz
	{0x07D9, 0x0856, 0x0966},  //869.350, D9 56 66 (allg. Nutzung, kein Kanalraster, 10mW, keine zeitl. Beschrnkung)
	{0x07d9, 0x0859, 0x0999},  //869.4MHz, D9 59 99
	{0x07e4, 0x08c0, 0x0900},  //915MHz
};

const word RFM69ConfigTbl[20] =
{
	0x0200,        //RegDataModul    FSK Packet
	0x0502,        //RegFdevMsb    241*61Hz = 35KHz
	0x0641,        //RegFdevLsb
	0x0334,        //RegBitrateMsb   32MHz/0x3410 = 2.4kpbs
	0x0410,        //RegBitrateLsb
	0x130F,        //RegOcp      Disable OCP
	0x1888,        //RegLNA            200R
	0x1952,        //RegRxBw     RxBW  83KHz
	0x2C00,        //RegPreambleMsb
	0x2D05,        //RegPreambleLsb  5Byte Preamble
	0x2E90,        //enable Sync.Word  2+1=3bytes
	0x2FAA,        //0xAA        SyncWord = aa2dd4
	0x302D,        //0x2D
	0x31D4,        //0xD4

	0x3700,        //RegPacketConfig1  Disable CRCNRZ encode
	0x3817,        //RegPayloadLength  23 bytes for length & Fixed length
	0x3C95,        //RegFiFoThresh

	0x581B,        //RegTestLna        Normal sensitivity
	//0x582D,        //RegTestLna        increase sensitivity with LNA (Note: consumption also increase!)
	0x6F30,        //RegTestDAGC       Improved DAGC
	//0x6F00,        //RegTestDAGC       Normal DAGC
	0x0104,        //Standby
};

const word RFM69RxTable[5] =
{
	 0x119F,        //RegPaLevel Fifo In for Rx
	 0x2544,        //DIO Mapping for Rx: DIO0=PayloadReady (benutzt), DIO1=FIFO-Level, DIO2=DATA, DIO3=FIFO Full
	 0x5A55,        //Normal and TRx
	 0x5C70,        //Normal and TRx
	 0x0110,        //Entry to Rx
};

const word RFM69TxTable[5] =
{
	0x2500,        //DIO Mapping for Tx -> DIO0 = Packet Sent (nicht genutzt, Abfrage via IRQFlags2-Register
	0x5A55,        //Normal and TRx
	0x5C70,        //Normal and TRx
	0x010C,        //Entry to Tx
};


/**********************************************************
**Name:     RFM69_Config
**Function: Initialize RFM69 & set it entry to standby mode
**Input:    none
**Output:   none
**********************************************************/
void RFM69_Config(void)
{
	byte i;
	for(i=0;i<3;i++)
	SPIWriteReg(RFM69FreqTbl[FREQ_SEL][i]);                 //Frequency parameter
	for(i=0;i<20;i++)                                       //base parameters
	SPIWriteReg(RFM69ConfigTbl[i]);
}

/**********************************************************
**Name:     RFM69_EntryRx
**Function: Set RFM69 entry Rx_mode
**Input:    None
**Output:   "0" for Error Status
**********************************************************/
int RFM69_EntryRx(void)
{
	byte i;
	//RFM69_Config();                                          //config RFM69 base parameters
	for(i=0;i<5;i++)                                         //config RFM69 RxMode parameters
	{
		SPIWriteReg(RFM69RxTable[i]);
	}
	_delay_ms(5);
	if((RFM69_ReadRegister(0x27,1) & 0xC0) == 0xC0)         // ModeReady und RxReady ?
	{
		return 1;
	}
	else
	{
		return 0;
	}
}

/**********************************************************
**Name:     RFM69_EntryTx
**Function: Set RFM69 entry Tx_mode
**Input:    None
**Output:   "0" for Error Status
**********************************************************/
byte RFM69_EntryTx(void)
{
	byte i;

// Benutztes Funkmodul am Anfang des Programms einstellen
#ifdef _RFM69W_
	// Sendeleistung einstellen (RFM69W - benutzt PA0):
	//RFM69_WriteRegister(0x11,0x90); // -2 dBm
	//RFM69_WriteRegister(0x11,0x92); //  0 dBm
	RFM69_WriteRegister(0x11,0x96); //  4 dBm
	//RFM69_WriteRegister(0x11,0x9A); //  8 dBm
	//RFM69_WriteRegister(0x11,0x9C); // 10 dBm
	//RFM69_WriteRegister(0x11,0x9F); // 13 dBm
#endif

#ifdef _RFM69HW_
	// Sendeleistung einstellen (RFM69HW - benutzt PA1 (PA2 und HighPower-Option nicht genutzt)):
	//RFM69_WriteRegister(0x11,0x50); // -2 dBm
	//RFM69_WriteRegister(0x11,0x52); //  0 dBm
	RFM69_WriteRegister(0x11,0x56); //  4 dBm
	//RFM69_WriteRegister(0x11,0x5A); //  8 dBm
	//RFM69_WriteRegister(0x11,0x5C); // 10 dBm
	//RFM69_WriteRegister(0x11,0x5F); // 13 dBm
#endif

	for(i=0;i<4;i++)                                         //config RFM69 TxMode parameters
	{
		SPIWriteReg(RFM69TxTable[i]);
	}
	_delay_ms(5);
	if((RFM69_ReadRegister(0x27,1) & 0xA0) == 0xA0)         // TxReady ?
	{
		return 1;
	}
	else
	{
		return 0;
	}
}

/**********************************************************
**Name:     RFM69_ClearFIFO
**Function: Change to RxMode from StandbyMode, can clear FIFO buffer
**Input:    None
**Output:   None
**********************************************************/
void RFM69_ClearFIFO(void)
{
	SPIWriteReg(0x0104);                                        //Entry Standby Mode
	SPIWriteReg(0x0110);                                        //Change to Rx Mode
}

/**********************************************************
**Name:     RFM69_Sleep
**Function: Set RFM69 to sleep mode
**Input:    none
**Output:   none
**********************************************************/
void RFM69_Sleep(void)
{
	SPIWriteReg(0x0100);                                        //sleep mode
}

/**********************************************************
**Name:     RFM69_Standby
**Function: Set RFM69 to Standby mode
**Input:    none
**Output:   none
**********************************************************/
void RFM69_Standby(void)
{
	SPIWriteReg(0x0104);                                        //standby mode
}

/**********************************************************
**Name:     RFM69_RxPacket
**Function: Check for receive one packet
**Input:    none
**Output:   "!0"-----Receive one packet
**          "0"------Nothing for receive
**********************************************************/
byte RFM69_RxPacket(void)
{
	byte i;

	// hier Abfrage auf INT, wenn Daten empfangen werden
	if(rx)
	{
		for(i=0;i<21;i++)
		RFM69_RxData[i] = 0x00;
		//SPIBurstRead(0x00, RFM69_RxData, 21);
		RFM69_ClearFIFO();
	}
	return(0);
}


/**********************************************************
**Name:     RFM69_TxPacket
**Function: send packet
**Input:    none
**Output:   none
**********************************************************/
void RFM69_TxPacket(byte *ptr, byte length)
{
	byte i;
	byte reg = 0 | WRITE;					// Null ist FIFO
	slaveSelect();
	SPI0_exchangeData(reg);					// SPI.transfer(reg);                   //Send register address FIFO
	for(i=0; i<length; i++)
	{
		SPI0_exchangeData(ptr[i]);            // Send data
		//USART0_sendChar(ptr[i]);			// Serial.print();
	}
	// warten, bis Paket gesendet...
	while((RFM69_ReadRegister(0x28,1)& 8) != 8);       // RegIrqFlags2, Bit 3
	slaveDeselect();
	USART0_sendString( (char *) " - Packet Sent\r\n");
}

//Read from register(s):
unsigned int RFM69_ReadRegister(byte thisRegister, int bytesToRead)
{
	byte inByte = 0;           // incoming byte from the SPI
	unsigned int result = 0;   // result to return

	// combine the address and the command into one byte
	byte dataToSend = thisRegister | READ;
	
	// take the chip select low to select the device:
	slaveSelect();											// *SS low - digitalWrite(chipSelectPin, LOW);
	_delay_us(50);											// delayMicroseconds(50);
	// send the device the register you want to read:
	SPI0_exchangeData(dataToSend);							//SPI.transfer(dataToSend);
	// send a value of 0 to read the first byte returned:
	result = SPI0_exchangeData(0x00);						// SPI.transfer(0x00);
	// decrement the number of bytes left to read:
	bytesToRead--;
	// if you still have another byte to read:
	if (bytesToRead > 0) 
	{
		// shift the first byte left, then get the second byte:
		result = result << 8;
		inByte = SPI0_exchangeData(0x00);					// SPI.transfer(0x00);
		// combine the byte you just got with the previous one:
		result = result | inByte;
		// decrement the number of bytes left to read:
		bytesToRead--;
	}
	// take the chip select high to de-select:
	_delay_us(50);								//delayMicroseconds(50);
	slaveDeselect();							// digitalWrite(chipSelectPin, HIGH);
	// return the result:
	return (result);
}


//Writes to a RFM69-Register
void RFM69_WriteRegister(byte thisRegister, byte thisValue)
{
	// combine the register address and the command into one byte:
	byte dataToSend = thisRegister | WRITE;

	// NSS low:
	slaveSelect();							// chipSelectPin, LOW);
	_delay_us(50);							// delayMicroseconds(10);
	SPI0_exchangeData(dataToSend);			// SPI.transfer(dataToSend); //Send register location
	SPI0_exchangeData(thisValue);			// SPI.transfer(thisValue);  //Send value to record into register
	_delay_us(50);							// delayMicroseconds(10);
	// NSS high:
	slaveDeselect();						// chipSelectPin, HIGH);
	_delay_us(50);							// delayMicroseconds(50);
}

// fills RFM69_RxData[] with RxFIFO
void RFM69_FifoRead(byte thisRegister, int bytesToRead)
{
	byte i = 0;
	
	// combine the address and the command into one byte
	byte dataToSend = thisRegister | READ;
	
	// take the chip select low to select the device:
	slaveSelect();
	_delay_us(50);
	
	// send the device the register you want to read:
	SPI0_exchangeData(dataToSend);
	// send a value of 0 to read the first byte returned:
	RFM69_RxData[i++] = SPI0_exchangeData(0x00);
	// decrement the number of bytes left to read:
	bytesToRead--;
	// if you still have another byte to read:
	while (bytesToRead > 0)
	{
		RFM69_RxData[i++] = SPI0_exchangeData(0x00);
		// decrement the number of bytes left to read:
		bytesToRead--;
	}
	// take the chip select high to de-select:
	_delay_us(50);
	slaveDeselect();
}

void SPIWriteReg(word Wort)
{
	byte reg = (Wort & 0xff00) >> 8;    // obere 8 Bits
	byte val = (Wort & 0xFF);           // untere 8 Bits
	//Serial.print("Reg: "); Serial.print(reg); Serial.print(" - val: "); Serial.println(val);
	RFM69_WriteRegister(reg, val);
	_delay_us(50);						// delayMicroseconds(50);
}
// =======================================================================================



// =======================================================================================
// -------------------------------- Setup ------------------------------------------------
// =======================================================================================

void setup() 
{
	//PORTMUX_CTRLA = 0;							// keine alternativen Portzuweisungen
	//PORTMUX_CTRLB = 0;
	
	CLKCTRL_init();
	USART0_init();									// 9600 Baud
	SPI0_init();

	// LED
	PORTC.DIRSET = PIN0_bm;							// PORTC.0 = OUTPUT (LED)

	// Interrupt-Pin fr RFM69 ist PC3:
	//PORTC.DIRCLR = PIN3_bm;							// Pin PC3 = input (PinChangeInterrupt)
	//PORTC.PIN3CTRL |= PORT_PULLUPEN_bm | PORT_ISC_BOTHEDGES_gc;
	
	// CPUINT.LVL0PRI = 0 << CPUINT_LVL0PRI_gp;		// Interrupt Level Priority: 0
	// CPUINT.LVL1VEC = 0x0A << CPUINT_LVL1VEC_gp;	// Interrupt Vector with High Priority: 0x08

	//PORTC_set_port_dir(PIN3_bm, PORT_DIR_IN);
	PORTC_set_pin_pull_mode(PIN3_bp, PORT_PULL_UP); 
	PORTC_pin_set_isc(PIN3_bp, PORT_ISC_BOTHEDGES_gc);		// Pin-Interrupt auf PortC.3 -> RFM69.DIO0
		
	// Spannung fr RFM69HW-Funkmodul einschalten
	PORTB.DIRSET = PIN1_bm;						// EN_Funk = Output
	PORTB.OUT |= PIN1_bm;						// EN_Funk = High
	_delay_ms(20);
	
	// jetzt Funkmodul initialisieren
	RFM69_Config();
	_delay_ms(20);
	
	USART0_sendString( (char *) " \r\n Setup Ready \r\n");
	
	sei();										// Global interrupts enable
}


// =======================================================================================
// =======================================================================================
// =======================================================================================


int main(void)
{

	setup();
	
	// Empfang einleiten
	while (RFM69_EntryRx()== 0)
	{
		USART0_sendString( (char *) "-RxEntry failed\r\n");
	}

	
	while (1)
	{
		// Test Schreiben und Lesen von RFM69-Registern - luft!
		//RFM69_WriteRegister(0x3E, 0x88);							// Testregister 1.Byte Encryption Key
		uint8_t value = RFM69_ReadRegister(0x10, 1);				// Versionsabfrage 0x24
		//uint8_t value = RFM69_ReadRegister(0x3E,1);					// sollte 0x88 = 136 sein
		debug_int(value);


		if(rx_daten | ( (RFM69_ReadRegister(0x28,1) & 0x04) == 0x04) )	// bei Empfang: Payload Ready?
		{
			if(rx_daten)
			{
				USART0_sendString( (char *) "INT detected\r\n");
				rx_daten = false;
			}
			
			RFM69_FifoRead(0x00,23);						// Register 0 (FIFO), 23 Bytes
			for(byte i=0; i<23;i++) USART0_sendChar(RFM69_RxData[i]);
			USART0_sendString( (char *) "\r\n");
			RFM69_ClearFIFO();
			//...
		}

		// String Senden
		while (RFM69_EntryTx()== 0)
		{
			USART0_sendString( (char *) "-TxEntry failed\r\n");
			_delay_ms(10);
		}
		
		RFM69_TxPacket( (byte *)RFM69_SendData, 23);

		RFM69_Config();
		
		while (RFM69_EntryRx()== 0)
		{
			USART0_sendString( (char *) "-RxEntry failed\r\n");
			_delay_ms(10);
		}
			
		PORTC.OUTTGL = PIN0_bm;				// Toggle PortC, Pin0 (LED)
		_delay_ms(5000);
	}
}

ISR(PORTC_PORT_vect)						// Interrupt Service fr PORTC-Pin-Change
{
	// here Code...
	rx_daten = true;
	PORTC.INTFLAGS |= PORT_INT3_bm  ;		// Clear Interrupt Pin Flag, must do this to handle next interrupt
}

