/*
 * AtTiny3216_TimerB_Standby_Mode.cpp
 * Test des Standby-Modes (zum Stromsparen) und TimerB0 Interrupt
 *
 * Programm zur Ermittlung des Stromverbrauchs des ATTiny3216 im Standby-Modus bei 32 kHz Takt (MAIN_CLK) unhd darunter
 * MAIN_CLK wird von 3,3 MHz auf den internen 32-kHz-Oszillator umgeschaltet und mit dem MAIN_CLK-Prescaler weiter geteilt.
 * Timer B toggelt LED an PortC.1. Der Prozessor luft zu 99,9% in Standby.
 * Lediglich zum Toggeln des PortC.1 wacht der Prozessor kurzzeitig auf.
 * Stromverbrauch: MAIN_CLK = 32,768 kHz, STANDBY-Modus: ca. 112A, wenn LED nicht leuchtet.
 *
 * TimerB bei 1 kHz MAIN_CLK bei Top-Wert 65500 (TCB0.CCMP = 65500) = 2 Minuten und 9 Sekunden bis zum Interrupt
 * TimerB bei 0,5 kHz MAIN_CLK bei Top-Wert 65500 (TCB0.CCMP = 65500) = 4 Minuten und 18 Sekunden bis zum Interrupt
 *  
 * luft!
 *
 * Created: 06.02.2020 11:12:52
 * Author : Micha
 */ 

// Einstellungen CLK_MAIN
#define F_CPU 3333333UL

#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <util/atomic.h>
#include <avr/sleep.h>
#include "MyPort.h"


// Variable im Interrupt benutzt
volatile uint8_t TCB0_isr_count;


void CLKCTRL_init(void)
{
	#if (F_CPU == 1000000UL)                     /* 1MHz, 16/16, set fuse for 16 MHz */
	_PROTECTED_WRITE(CLKCTRL.MCLKCTRLB, CLKCTRL_PDIV_16X_gc | CLKCTRL_PEN_bm);
	#endif
	
	#if (F_CPU == 20000000UL)                  /* 20MHz, no division */
	_PROTECTED_WRITE(CLKCTRL.MCLKCTRLB, 0 << CLKCTRL_PEN_bp);
	#endif
	
	#if (F_CPU == 3333333UL)					/* 3.3 MHz bei Fuse = 20 MHz / 6 = 3.3 MHz*/
	_PROTECTED_WRITE(CLKCTRL.MCLKCTRLB, CLKCTRL_PDIV_6X_gc | CLKCTRL_PEN_bm);  /* default 3.33MHz (20MHz/6) set fuse for 20MHz */
	#endif
}

// schaltet die MAIN_CLK auf internen Oszillator 32,768 kHz
void Main_Clock_32_kHz(void)
{
	_PROTECTED_WRITE(CLKCTRL.MCLKCTRLA, CLKCTRL_CLKSEL_OSCULP32K_gc);	// MAIN_CLK = 32,768 kHz
	_PROTECTED_WRITE(CLKCTRL.MCLKCTRLB, 0 << CLKCTRL_PEN_bp);	// Prescaler aus
}

// schaltet die MAIN_CLK auf internen Oszillator 32 kHz mit Prescaler 32 = 32 / 32 = 1 kHz
void Main_Clock_1_kHz(void)
{
	_PROTECTED_WRITE(CLKCTRL.MCLKCTRLA, CLKCTRL_CLKSEL_OSCULP32K_gc);	// MAIN_CLK = 32,768 kHz
	_PROTECTED_WRITE(CLKCTRL.MCLKCTRLB, CLKCTRL_PDIV_32X_gc | CLKCTRL_PEN_bm);

}

// schaltet die MAIN_CLK auf internen Oszillator 32 kHz mit Prescaler 32 = 32 / 64 = 0.5 kHz
void Main_Clock_0_5_kHz(void)
{
	_PROTECTED_WRITE(CLKCTRL.MCLKCTRLA, CLKCTRL_CLKSEL_OSCULP32K_gc);	// MAIN_CLK = 32,768 kHz
	_PROTECTED_WRITE(CLKCTRL.MCLKCTRLB, CLKCTRL_PDIV_64X_gc | CLKCTRL_PEN_bm);

}

// schaltet die Main_CLK auf intern 16 bzw. 20 MHz, je nach Einstellung der FUSE
void Main_Clock_20_MHz(void)
{
	_PROTECTED_WRITE(CLKCTRL.MCLKCTRLA, CLKCTRL_CLKSEL_OSC20M_gc);
}

// schaltet die Main_CLK auf intern 3,33333 MHz, bei FUSE = 20 MHz / Prescaler = 6 (20 MHz / 6)
void Main_Clock_3_3_MHz(void)
{
	_PROTECTED_WRITE(CLKCTRL.MCLKCTRLA, CLKCTRL_CLKSEL_OSC20M_gc);
	_PROTECTED_WRITE(CLKCTRL.MCLKCTRLB, CLKCTRL_PDIV_6X_gc | CLKCTRL_PEN_bm);  /* default 3.33MHz (20MHz/6) set fuse for 20MHz */

}

void io_init()
{
	// Strom sparen durch das Ausschalten des INPUT_BUFFERS fr nicht genutzte Pins
	PORTA_pin_set_isc(PIN0_bm, PORT_ISC_INPUT_DISABLE_gc);
	PORTA_pin_set_isc(PIN1_bm, PORT_ISC_INPUT_DISABLE_gc);
	PORTA_pin_set_isc(PIN2_bm, PORT_ISC_INPUT_DISABLE_gc);
	PORTA_pin_set_isc(PIN3_bm, PORT_ISC_INPUT_DISABLE_gc);
	PORTA_pin_set_isc(PIN4_bm, PORT_ISC_INPUT_DISABLE_gc);
	PORTA_pin_set_isc(PIN5_bm, PORT_ISC_INPUT_DISABLE_gc);
	PORTA_pin_set_isc(PIN6_bm, PORT_ISC_INPUT_DISABLE_gc);
	PORTA_pin_set_isc(PIN7_bm, PORT_ISC_INPUT_DISABLE_gc);
	
	PORTB_pin_set_isc(PIN0_bm, PORT_ISC_INPUT_DISABLE_gc);
	PORTB_pin_set_isc(PIN1_bm, PORT_ISC_INPUT_DISABLE_gc);
	PORTB_pin_set_isc(PIN2_bm, PORT_ISC_INPUT_DISABLE_gc);
	PORTB_pin_set_isc(PIN3_bm, PORT_ISC_INPUT_DISABLE_gc);
	//PORTB_pin_set_isc(PIN4_bm, PORT_ISC_INPUT_DISABLE_gc);
	PORTB_pin_set_isc(PIN5_bm, PORT_ISC_INPUT_DISABLE_gc);
	PORTB_pin_set_isc(PIN6_bm, PORT_ISC_INPUT_DISABLE_gc);
	PORTB_pin_set_isc(PIN7_bm, PORT_ISC_INPUT_DISABLE_gc);

	// PortC.1 wird benutzt...weiter gehts mit Pin2
	PORTC_pin_set_isc(PIN0_bm, PORT_ISC_INPUT_DISABLE_gc);
	//PORTC_pin_set_isc(PIN1_bm, PORT_ISC_INPUT_DISABLE_gc);
	PORTC_pin_set_isc(PIN2_bm, PORT_ISC_INPUT_DISABLE_gc);
	PORTC_pin_set_isc(PIN3_bm, PORT_ISC_INPUT_DISABLE_gc);
	PORTC_pin_set_isc(PIN4_bm, PORT_ISC_INPUT_DISABLE_gc);
	PORTC_pin_set_isc(PIN5_bm, PORT_ISC_INPUT_DISABLE_gc);
	PORTC_pin_set_isc(PIN6_bm, PORT_ISC_INPUT_DISABLE_gc);
	PORTC_pin_set_isc(PIN7_bm, PORT_ISC_INPUT_DISABLE_gc);

	// LED
	PORTC.DIRSET = PIN1_bm;						// PORTC.1 = OUTPUT (LED)

}


// ===========================================================================================
// TimerB - Periodic interrupt mode
// ===========================================================================================

// nderungen der vordefinierten Parameter in Visual Studio /Atmel Studio 7:
// Mit Maus Wort, z.B. TCB_CLKSEL_CLKDIV1_gc markieren, dann rechte Maustaste, dann
// "Goto Implementation" aufrufen (oder ALT-G). Im nun zu sehenden Text sind alternative
// Einstellungen zu sehen.

// TimerB0 (TBC0) interrupt tick.
// Periodic interrupt mode
void timerB0_init(void)
{

    /*  By default, TCB is in Periodic interrupt mode
        In the periodic interrupt mode the counter counts to the capture value and restarts from zero. Interrupt is
        generated when counter is equal to TOP.
    */

    // Set the TOP value in the Compare/Capture register (TCB.CCMP)
    TCB0.CCMP = 65500;

    /*
        Enable the counter by writing a '1' to the ENABLE bit in the Control A register (TCB.CTRLA).
        The counter will start counting clock ticks according to the prescaler setting in the Clock Select bit
        field (CLKSEL in TCB.CTRLA).
    */
    //TCB0.CTRLA = TCB_CLKSEL_CLKDIV1_gc;			// Prescaler = 0
	TCB0.CTRLA = TCB_CLKSEL_CLKDIV2_gc;				// Prescaler = CLK_PER / 2
	TCB0.CTRLA |= 1 << TCB_ENABLE_bp;				// Enable TimerB0
	TCB0.CTRLA |= 1 << TCB_RUNSTDBY_bp;				// dann luft der Timer auch in STANDBY
	
    // Enable the capture interrupt
    TCB0.INTCTRL = 1 << TCB_CAPT_bp;

	//sei();											// nicht vergessen!

}



// =====================================================================================



int main(void)
{
	CLKCTRL_init();
	io_init();
	timerB0_init();
	Main_Clock_0_5_kHz();
	
	set_sleep_mode(SLEEP_MODE_STANDBY);
	sei();											// nicht vergessen!

	
    /* Replace with your application code */
    while (1) 
    {
		//PORTC.OUTTGL = PIN1_bm;
		//_delay_ms(50); 
		;                                                   
    }
}


// -- ISR ---------------------------------------------------------------------------

ISR(TCB0_INT_vect)
{
	/* Insert your TCB interrupt handling code */
	 extern volatile uint8_t TCB0_isr_count;

	 TCB0_isr_count++;
	 PORTC.OUTTGL = PIN1_bm;				// Toggle PortC, Pin1 (LED)

	/**
	 * The interrupt flag is cleared by writing 1 to it, or when the Capture register
	 * is read in Capture mode
	 */
	TCB0.INTFLAGS = TCB_CAPT_bm;
	sleep_mode();							// go to sleep again
}